from __future__ import annotations

import json
from dataclasses import dataclass
from typing import Any, Tuple

import geopandas as gpd
import matplotlib.patches as patches
import matplotlib.pyplot as plt
import numpy as np
from shapely.geometry import LineString, Point, Polygon

from pyvibracore.results.plot_utils import _north_arrow, _scalebar


@dataclass(frozen=True)
class VibrationResults:
    """
    Dataclass that holds the information from `/cur166/validation/multi` or `/prepal/validation/multi`

    Attributes
    -----------
    gdf: gpd.GeoDataFrame

    """

    gdf: gpd.GeoDataFrame

    @classmethod
    def from_api_response(cls, response_dict: dict) -> "VibrationResults":
        """
        Stores the response of the VibraCore endpoint

        Parameters
        ----------
        response_dict:
           The resulting response of a call to `/cur166/validation/multi` or `/prepal/validation/multi`
        """
        return cls(
            gpd.read_file(json.dumps(response_dict), driver="GeoJSON").set_crs(
                "EPSG:28992", allow_override=True
            )
        )

    def map(
        self,
        source_location: Point | LineString | Polygon,
        title: str = "Legend:",
        figsize: Tuple[float, float] = (10.0, 12.0),
        settings: dict | None = None,
        **kwargs: Any,
    ) -> plt.Figure:
        """
        Create map plot of the results

        Parameters
        ----------
        source_location:
            location of the vibration source
        title:
            Legend title
        figsize:
            Size of the activate figure, as the `plt.figure()` argument.
        settings:
            Plot settings used in plot: default settings are:

            .. code-block:: python

                {
                    "source_location": {"label": "Trillingsbron", "color": "black"},
                    "insufficient_cat1": {
                        "label": "Voldoet Niet - Cat.1",
                        "color": "orange",
                    },
                    "insufficient_cat2": {"label": "Voldoet Niet - Cat.2", "color": "red"},
                    "sufficient": {"label": "Voldoet", "color": "green"},
                }
        **kwargs:
            All additional keyword arguments are passed to the `pyplot.subplots()` call.

        Returns
        -------
        Figure
        """
        if settings is None:
            settings = {
                "source_location": {"label": "Trillingsbron", "color": "black"},
                "insufficient_cat1": {
                    "label": "Voldoet Niet - Cat.1",
                    "color": "orange",
                },
                "insufficient_cat2": {"label": "Voldoet Niet - Cat.2", "color": "red"},
                "sufficient": {"label": "Voldoet", "color": "green"},
            }

        kwargs_subplot = {
            "figsize": figsize,
            "tight_layout": True,
        }

        kwargs_subplot.update(kwargs)

        fig, axes = plt.subplots(**kwargs_subplot)

        gpd.GeoSeries(source_location).plot(
            ax=axes,
            color=settings["source_location"]["color"],
            alpha=1,
            zorder=1,
            aspect=1,
        )

        # plot category 1 zone of influence
        self.gdf.where(
            np.logical_and(self.gdf["cat"] == "one", ~self.gdf["check"])
        ).plot(
            ax=axes, zorder=2, color=settings["insufficient_cat1"]["color"], aspect=1
        )
        self.gdf.where(
            np.logical_and(self.gdf["cat"] == "two", ~self.gdf["check"])
        ).plot(
            ax=axes, zorder=2, color=settings["insufficient_cat2"]["color"], aspect=1
        )
        self.gdf.where(self.gdf.check).plot(
            ax=axes, zorder=2, color=settings["sufficient"]["color"], aspect=1
        )
        self.gdf.where(self.gdf.check).buffer(self.gdf.x_required).plot(
            ax=axes, alpha=0.25, zorder=1, aspect=1
        )
        self.gdf.where(~self.gdf.check).buffer(self.gdf.x_required).plot(
            ax=axes, alpha=0.6, zorder=1, aspect=1
        )

        for idx, row in self.gdf.iterrows():
            x = row.geometry.centroid.xy[0][0]
            y = row.geometry.centroid.xy[1][0]

            axes.annotate(
                idx,
                xy=(x, y),
                horizontalalignment="center",
            )

        # add legend
        axes.legend(
            title=title,
            title_fontsize=18,
            fontsize=15,
            loc="lower right",
            handles=[
                patches.Patch(
                    facecolor=value["color"],
                    label=value["label"],
                    alpha=0.9,
                    linewidth=2,
                    edgecolor="black",
                )
                for value in settings.values()
            ],
        )

        _north_arrow(axes)
        _scalebar(axes)

        return fig


def map_payload(
    gdf: gpd.GeoDataFrame,
    source_location: Point | LineString | Polygon,
    title: str = "Legend:",
    figsize: Tuple[float, float] = (10.0, 12.0),
    settings: dict | None = None,
    **kwargs: Any,
) -> plt.Figure:
    """
    Create map of the input building settings.

    Parameters
    ----------
    gdf:
        GeoDataFrame of the input buildings
    source_location:
        location of the vibration source
    title:
        Legend title
    figsize:
        Size of the activate figure, as the `plt.figure()` argument.
    settings:
        Plot settings used in plot: default settings are:

        .. code-block:: python

            {
                "source_location": {"label": "Trillingsbron", "color": "black"},
                "sensitive_cat1": {
                    "label": "Monumentaal/ gevoelig - Cat.1",
                    "color": "blue",
                },
                "sensitive_cat2": {
                    "label": "Monumentaal/ gevoelig - Cat.2",
                    "color": "cyan",
                },
                "normal_cat1": {"label": "Normaal - Cat.1", "color": "orange"},
                "normal_cat2": {"label": "Normaal - Cat.2", "color": "olive"},
            }

    **kwargs:
        All additional keyword arguments are passed to the `pyplot.subplots()` call.

    Returns
    -------
    Figure
    """
    if settings is None:
        settings = {
            "source_location": {"label": "Trillingsbron", "color": "black"},
            "sensitive_cat1": {
                "label": "Monumentaal/ gevoelig - Cat.1",
                "color": "blue",
            },
            "sensitive_cat2": {
                "label": "Monumentaal/ gevoelig - Cat.2",
                "color": "cyan",
            },
            "normal_cat1": {"label": "Normaal - Cat.1", "color": "orange"},
            "normal_cat2": {"label": "Normaal - Cat.2", "color": "olive"},
        }

    kwargs_subplot = {
        "figsize": figsize,
        "tight_layout": True,
    }

    kwargs_subplot.update(kwargs)

    fig, axes = plt.subplots(**kwargs_subplot)

    gpd.GeoSeries(source_location).plot(
        ax=axes, color=settings["source_location"]["color"], alpha=1, zorder=1, aspect=1
    )

    gdf.where(
        np.logical_and(
            gdf["category"] == "one",
            np.logical_or(gdf["monumental"], gdf["vibrationSensitive"]),
        )
    ).plot(ax=axes, zorder=2, color=settings["sensitive_cat1"]["color"], aspect=1)
    gdf.where(
        np.logical_and(
            gdf["category"] == "one",
            ~np.logical_or(gdf["monumental"], gdf["vibrationSensitive"]),
        )
    ).plot(ax=axes, zorder=2, color=settings["normal_cat1"]["color"], aspect=1)
    gdf.where(
        np.logical_and(
            gdf["category"] == "two",
            np.logical_or(gdf["monumental"], gdf["vibrationSensitive"]),
        )
    ).plot(ax=axes, zorder=2, color=settings["sensitive_cat2"]["color"], aspect=1)
    gdf.where(
        np.logical_and(
            gdf["category"] == "two",
            ~np.logical_or(gdf["monumental"], gdf["vibrationSensitive"]),
        )
    ).plot(ax=axes, zorder=2, color=settings["normal_cat2"]["color"], aspect=1)

    for idx, row in gdf.iterrows():
        x = row.geometry.centroid.xy[0][0]
        y = row.geometry.centroid.xy[1][0]

        axes.annotate(
            idx,
            xy=(x, y),
            horizontalalignment="center",
        )

    # add legend
    axes.legend(
        title=title,
        title_fontsize=18,
        fontsize=15,
        loc="lower right",
        handles=[
            patches.Patch(
                facecolor=value["color"],
                label=value["label"],
                alpha=0.9,
                linewidth=2,
                edgecolor="black",
            )
            for value in settings.values()
        ],
    )

    _north_arrow(axes)
    _scalebar(axes)

    return fig


def plot_reduction(
    response_dict: dict,
    sensitive: bool = False,
    figsize: Tuple[float, float] = (8, 8),
    **kwargs: Any,
) -> plt.Figure:
    """
    PLot single vibration prediction reduction plot

    Parameters
    ----------
    response_dict:
        response of the single prepal or cur166 endpoint.
    sensitive:
        Default is False
        Flag that indicates if vibration sensitive results are included
    figsize:
        Size of the activate figure, as the `plt.figure()` argument.
    **kwargs:
        All additional keyword arguments are passed to the `pyplot.subplots()` call.

    Returns
    -------

    """
    kwargs_subplot = {
        "figsize": figsize,
        "tight_layout": True,
    }

    kwargs_subplot.update(kwargs)

    fig, axes = plt.subplots(**kwargs_subplot)

    axes.axvline(
        x=response_dict["calculation"]["distance"],
        linestyle="-",
        label="Building distance",
        color="green",
    )

    # normal
    axes.axhline(
        y=response_dict["calculation"]["failureValueVibrationVelocity"],
        linestyle="-",
        label="Vr",
        color="orange",
    )
    axes.axvline(
        x=response_dict["calculation"]["distanceRequired"],
        linestyle="-",
        label="Distance required",
        color="black",
    )
    axes.plot(
        response_dict["data"]["distance"],
        response_dict["data"]["vibrationVelocity"],
        linestyle="-",
        label="Vd",
        color="blue",
    )

    # sensitive
    if sensitive:
        axes.axhline(
            y=response_dict["calculation"][
                "failureValueVibrationVelocityVibrationSensitive"
            ],
            linestyle="--",
            label="$Vr_{sensitive}$",
            color="orange",
        )
        axes.axvline(
            x=response_dict["calculation"][
                "distanceRequiredVelocityVibrationSensitive"
            ],
            linestyle="--",
            label="Distance required",
            color="black",
        )
        # excitation
        axes.axhline(
            y=response_dict["calculation"]["failureValueExcitationVelocity"],
            linestyle="-.",
            label="$Vr_{velocity}$",
            color="orange",
        )
        axes.axvline(
            x=response_dict["calculation"]["distanceRequiredExcitationVelocity"],
            linestyle="-.",
            label="Distance required",
            color="black",
        )
        axes.plot(
            response_dict["data"]["distance"],
            response_dict["data"]["vibrationVelocityVibrationSensitive"],
            linestyle="--",
            label="Vd",
            color="blue",
        )

    axes.set_xlabel("Distance from source [m]")
    axes.set_ylabel("Vibration velocity [mm/s]")
    axes.set_xlim(0, 50)
    axes.set_ylim(0, 30)
    axes.legend(
        title=f"Vibration prediction: {response_dict.get('ID')}",
        title_fontsize=12,
        fontsize=9,
        loc="upper right",
    )

    return fig



from copy import deepcopy
from dataclasses import dataclass
from typing import Dict, List, Literal

import numpy as np
from pygef.cpt import CPTData

from .constants import SHEETPILE_REFERENCE_PROFILES

CustomInterval = 0.5
UnitWeightWater = 9.81


@dataclass(frozen=True)
class VibrationSource:
    """

    Attributes
    -----------
    areaShaftSpecific: float
        Specific shaft area of the single sheet pile (1 sheet) per unit length [m^2/m].
    areaTipSpecific: float
        Specific tip area of the single sheet pile (1 sheet) [m^2].
    amountOfSheetPiles: int = 1
        Number of sheets
            - push = 1
            - vibrate = 2
    slotResistanceSpecific: float
        Specific sheet pile slot resistance [kN/m] based on CUR 166 6th edition.
        for new sheet piles:
            - push = 5
            - vibrate = 10
    sheetPileName: str = "User defined"
        Sheet pile name
    """

    areaShaftSpecific: float
    areaTipSpecific: float
    slotResistanceSpecific: float
    amountOfSheetPiles: int = 1
    sheetPileName: str = "User defined"

    @classmethod
    def from_sheet_pile_name(
        cls,
        name: str,
        amount_of_sheet_piles: int = 1,
        slot_resistance_specific: float = 10.0,
    ) -> "VibrationSource":
        """

        Parameters
        ----------
        name: str
            Sheet pile name
        amount_of_sheet_piles: int = 1
            Number of sheets
        slot_resistance_specific: float = 10
            Specific sheet pile slot resistance [kN/m] based on CUR 166 6th edition.
        Returns
        -------
        VibrationSource
        """
        props = next(
            (item for item in SHEETPILE_REFERENCE_PROFILES if item["label"] == name),
            None,
        )
        if props is None:
            raise ValueError(
                f"{name} is not a valid sheet pile name. "
                "Please specify the areaShaftSpecific and areaTipSpecific in the "
                "main class or update the SHEETPILE_REFERENCE_PROFILES table."
            )

        return cls(
            areaShaftSpecific=props.get("area_shaft_specific"),  # type: ignore
            areaTipSpecific=props.get("area_tip_specific"),  # type: ignore
            amountOfSheetPiles=amount_of_sheet_piles,
            slotResistanceSpecific=slot_resistance_specific,
            sheetPileName=name,
        )


def create_multi_cpt_impact_force_payload(
    cptdata_objects: List[CPTData],
    classify_tables: Dict[str, dict],
    vibration_source: VibrationSource,
    friction_strategy: Literal["CPTFrictionStrategy", "SlipFrictionStrategy"],
    drive_strategy: Literal["vibrate", "push"],
    installation_level_offset: float,
    zeta: float = 0.6,
) -> dict:
    """
    Creates a dictionary with the payload content for the VibraCore endpoint
    "/impact-force/calculation/multi"

    This dictionary can be passed directly to `nuclei.client.call_endpoint()`.

    Parameters
    ----------
    cptdata_objects:
        A list of pygef.CPTData objects
    classify_tables:
        A dictionary, mapping `CPTData.alias` values to dictionary with the resulting response
        of a call to CPTCore `classify/*` information, containing the following keys:

            - geotechnicalSoilName: Sequence[str]
                Geotechnical Soil Name related to the ISO
            - lowerBoundary: Sequence[float]
                Lower boundary of the layer [m]
            - upperBoundary: Sequence[float]
                Upper boundary of the layer [m]
            - color: Sequence[str]
                Hex color code
            - mainComponent: Sequence[Literal["rocks", "gravel", "sand", "silt", "clay", "peat"]]
                Main soil component
            - cohesion: Sequence[float]
                Cohesion of the layer [kPa]
            - gamma_sat: Sequence[float]
                Saturated unit weight [kN/m^3]
            - gamma_unsat: Sequence[float]
                Unsaturated unit weight [kN/m^3]
            - phi: Sequence[float]
                Phi [degrees]
            - undrainedShearStrength: Sequence[float]
                Undrained shear strength [kPa]
    vibration_source: VibrationSource
        Vibration source object
    friction_strategy: Literal["CPTFrictionStrategy", "SlipFrictionStrategy"]
        Defines the strategy on how to compute the sleeve friction:

            - CPTFrictionStrategy

        The CPT Friction Strategy takes the sleeve friction provided by the cpt.
        Note that this method is only valid when the sleeve friction is provided within
        the cpt object.

            - SlipFrictionStrategy

        The Slip Friction Strategy calculated the sleeve friction based on the gamma,
        gamma saturation, phi and the undrained shear strength. To use this methode the
        values must be provided within the cpt object.
    drive_strategy: Literal["vibrate", "push"]
        Defines the strategy on how to compute the impact force:

            - vibrate

        The vibration strategy calculates the impact force based on a vibration installation method.

            - push

        The push strategy calculates the impact force based on a push installation method.
    installation_level_offset: float
        Installation level of the sheet pile [m w.r.t REF]
    zeta: float = 0.6
        verknedingsfactor [-], used in the push drive strategy based on CUR 166 6th edition.

    Returns
    -------
    payload: dict
    """

    payload = {
        "soilProperties": [
            {
                "cptObject": {
                    "coneResistance": cpt.data.get_column("coneResistance")
                    .clip(lower_bound=0, upper_bound=1e10)
                    .to_list(),
                    "depthOffset": cpt.data.get_column("depthOffset").to_list(),
                    "localFriction": cpt.data.get_column("localFriction")
                    .clip(lower_bound=0, upper_bound=1e10)
                    .to_list(),
                    "name": cpt.alias,
                    "verticalPositionOffset": cpt.delivered_vertical_position_offset,
                    "x": cpt.delivered_location.x,
                    "y": cpt.delivered_location.y,
                },
                "customInterval": CustomInterval,
                "groundwaterLevelOffset": cpt.groundwater_level_offset
                if cpt.groundwater_level_offset
                else cpt.delivered_vertical_position_offset - 1,
                "layerTable": {
                    "gamma_sat": classify_tables[cpt.alias].get("gamma_sat"),
                    "gamma_unsat": classify_tables[cpt.alias].get("gamma_unsat"),
                    "phi": classify_tables[cpt.alias].get("phi"),
                    "soilcode": classify_tables[cpt.alias].get("mainComponent"),
                    "undrainedShearStrength": classify_tables[cpt.alias].get(
                        "undrainedShearStrength"
                    ),
                    "upperBoundary": (
                        cpt.delivered_vertical_position_offset
                        - np.array(classify_tables[cpt.alias].get("upperBoundary"))
                    ).tolist(),
                },
                "unitWeightWater": UnitWeightWater,
            }
            for cpt in cptdata_objects
        ],
        "vibrationSource": {
            "areaShaftSpecific": vibration_source.areaShaftSpecific,
            "areaTipSpecific": vibration_source.areaTipSpecific,
            "numSheets": vibration_source.amountOfSheetPiles,
            "sheetPileName": vibration_source.sheetPileName,
            "slotResistanceSpecific": vibration_source.slotResistanceSpecific,
        },
        "frictionStrategy": friction_strategy,
        "impactForceCalculation": {
            "driveStrategy": drive_strategy,
            "installationLevel": installation_level_offset,
            "zeta": zeta,
        },
    }
    return payload


def create_multi_cpt_impact_force_report_payload(
    multi_cpt_payload: dict,
    project_name: str,
    project_id: str,
    author: str,
) -> dict:
    """
    Creates a dictionary with the payload content for the VibraCore endpoint
    "/impact-force/report"

    This dictionary can be passed directly to `nuclei.client.call_endpoint()`.

    Parameters
    ----------
    multi_cpt_payload:
        The result of a call to `create_multi_cpt_impact_force_payload()`
    project_name:
        The name of the project.
    project_id:
        The identifier (code) of the project.
    author:
        The author of the report.

    Returns
    -------
    payload: dict
    """
    payload = deepcopy(multi_cpt_payload)
    payload.update(
        dict(
            reportProperties=dict(
                author=author,
                projectNumber=project_id,
                projectName=project_name,
            ),
        )
    )
    return payload


def create_single_cpt_impact_force_payload(multi_cpt_payload: dict, name: str) -> dict:
    """
    Creates a dictionary with the payload content for the VibraCore endpoint
    "/impact-force/calculation/single"

    This dictionary can be passed directly to `nuclei.client.call_endpoint()`.

    Parameters
    ----------
    multi_cpt_payload: dict
        The result of a call to `create_multi_cpt_impact_force_payload()`
    name: str
        CPT name

    Returns
    -------
    payload: dict
    """
    payload = deepcopy(multi_cpt_payload)

    props = next(
        (
            item
            for item in payload["soilProperties"]
            if item["cptObject"]["name"] == name
        ),
        None,
    )
    if props is None:
        raise ValueError(f"{name} is not a valid CPT name.")
    payload.update(dict(soilProperties=props))

    return payload



import matplotlib.pyplot as plt

from pyvibracore.input.vibration_properties import (
    BAG_WFS_URL,
    get_buildings_geodataframe,
)
from pyvibracore.results.sound_result import get_normative_building, map_sound


def test_map_sound(requests_mock, mock_bag_response, mock_source_location):
    requests_mock.get(BAG_WFS_URL, json=mock_bag_response)
    gdf = get_buildings_geodataframe(1, 2, 3, 4)

    fig = map_sound(
        gdf,
        source_location=mock_source_location,
        building_name="0",
        power=140,
        k2=5,
        period=5,
    )

    assert isinstance(fig, plt.Figure)


def test_get_normative_building(requests_mock, mock_bag_response, mock_source_location):
    requests_mock.get(BAG_WFS_URL, json=mock_bag_response)
    gdf = get_buildings_geodataframe(1, 2, 3, 4)

    name = get_normative_building(
        gdf,
        location=mock_source_location,
    )

    assert isinstance(name, str)



SHEETPILE_REFERENCE_PROFILES = [
    {"label": "AZ12-770", "area_tip_specific": 0.00925, "area_shaft_specific": 1.86},
    {"label": "AZ13-770", "area_tip_specific": 0.00969, "area_shaft_specific": 1.86},
    {"label": "AZ14-770", "area_tip_specific": 0.01013, "area_shaft_specific": 1.86},
    {"label": "AZ12-700", "area_tip_specific": 0.00862, "area_shaft_specific": 1.72},
    {"label": "AZ13-700", "area_tip_specific": 0.00943, "area_shaft_specific": 1.72},
    {"label": "AZ14-700", "area_tip_specific": 0.01023, "area_shaft_specific": 1.72},
    {"label": "AZ17-700", "area_tip_specific": 0.00931, "area_shaft_specific": 1.86},
    {"label": "AZ18-700", "area_tip_specific": 0.00975, "area_shaft_specific": 1.86},
    {"label": "AZ19-700", "area_tip_specific": 0.01019, "area_shaft_specific": 1.86},
    {"label": "AZ20-700", "area_tip_specific": 0.01064, "area_shaft_specific": 1.86},
    {"label": "AZ24-700", "area_tip_specific": 0.01219, "area_shaft_specific": 1.94},
    {"label": "AZ26-700", "area_tip_specific": 0.0131, "area_shaft_specific": 1.94},
    {"label": "AZ28-700", "area_tip_specific": 0.01402, "area_shaft_specific": 1.94},
    {"label": "AZ36-700N", "area_tip_specific": 0.01511, "area_shaft_specific": 2.06},
    {"label": "AZ38-700N", "area_tip_specific": 0.0161, "area_shaft_specific": 2.06},
    {"label": "AZ40-700N", "area_tip_specific": 0.01709, "area_shaft_specific": 2.06},
    {"label": "AZ42-700N", "area_tip_specific": 0.01811, "area_shaft_specific": 2.06},
    {"label": "AZ44-700N", "area_tip_specific": 0.0191, "area_shaft_specific": 2.06},
    {"label": "AZ46-700N", "area_tip_specific": 0.02009, "area_shaft_specific": 2.06},
    {"label": "AZ48-700", "area_tip_specific": 0.02019, "area_shaft_specific": 2.04},
    {"label": "AZ50-700", "area_tip_specific": 0.02118, "area_shaft_specific": 2.04},
    {"label": "AZ52-700", "area_tip_specific": 0.02217, "area_shaft_specific": 2.04},
    {"label": "PU18-1", "area_tip_specific": 0.00925, "area_shaft_specific": 1.74},
    {"label": "PU18+1", "area_tip_specific": 0.01034, "area_shaft_specific": 1.74},
    {"label": "PU22-1", "area_tip_specific": 0.01043, "area_shaft_specific": 1.80},
    {"label": "PU22+1", "area_tip_specific": 0.01152, "area_shaft_specific": 1.80},
    {"label": "PU28-1", "area_tip_specific": 0.01241, "area_shaft_specific": 1.86},
    {"label": "PU28+1", "area_tip_specific": 0.01353, "area_shaft_specific": 1.86},
    {"label": "PU32-1", "area_tip_specific": 0.014, "area_shaft_specific": 1.84},
    {"label": "PU32+1", "area_tip_specific": 0.01508, "area_shaft_specific": 1.84},
    {"label": "PU12", "area_tip_specific": 0.00842, "area_shaft_specific": 1.60},
    {"label": "PU12S", "area_tip_specific": 0.00905, "area_shaft_specific": 1.60},
    {"label": "PU22", "area_tip_specific": 0.01097, "area_shaft_specific": 1.80},
    {"label": "PU18", "area_tip_specific": 0.0098, "area_shaft_specific": 1.74},
    {"label": "PU32", "area_tip_specific": 0.01454, "area_shaft_specific": 1.84},
    {"label": "PU28", "area_tip_specific": 0.01297, "area_shaft_specific": 1.86},
]

SOIL_REFERENCE = [
    # CUR 166-1997 Tabel 5.16 parameters voor het inheien buispalen
    {
        "location": "Amsterdam",
        "method": "driving",
        "vibration_direction": "vertical",
        "Uo": 0.03,
        "alpha": 0.03,
        "Vo": 0.6,
    },
    {
        "location": "Maasvlakte",
        "method": "driving",
        "vibration_direction": "vertical",
        "Uo": 0.04,
        "alpha": 0.02,
        "Vo": 0.6,
    },
    {
        "location": "Rotterdam",
        "method": "driving",
        "vibration_direction": "vertical",
        "Uo": 0.017,
        "alpha": 0.03,
        "Vo": 0.6,
    },
    {
        "location": "Rotterdam",
        "method": "driving",
        "vibration_direction": "horizontal",
        "Uo": 0.026,
        "alpha": 0.03,
        "Vo": 0.6,
    },
    # CUR 166-1997 Tabel 5.17 Parameters voor het intrillen van stale planken (tot 14 meter)
    {
        "location": "Amsterdam",
        "method": "vibrate",
        "vibration_direction": "vertical",
        "Uo": 1.1,
        "alpha": 0.02,
        "Vo": 0.9,
    },
    {
        "location": "Amsterdam",
        "method": "vibrate",
        "vibration_direction": "horizontal",
        "Uo": 1.6,
        "alpha": 0.02,
        "Vo": 1.5,
    },
    {
        "location": "Eindhoven",
        "method": "vibrate",
        "vibration_direction": "vertical",
        "Uo": 1.9,
        "alpha": 0.02,
        "Vo": 1.1,
    },
    {
        "location": "Eindhoven",
        "method": "vibrate",
        "vibration_direction": "horizontal",
        "Uo": 2.6,
        "alpha": 0.02,
        "Vo": 0.8,
    },
    {
        "location": "Groningen",
        "method": "vibrate",
        "vibration_direction": "vertical",
        "Uo": 1.7,
        "alpha": 0.02,
        "Vo": 1.8,
    },
    {
        "location": "Groningen",
        "method": "vibrate",
        "vibration_direction": "horizontal",
        "Uo": 0.9,
        "alpha": 0.02,
        "Vo": 0.5,
    },
    {
        "location": "Den Haag",
        "method": "vibrate",
        "vibration_direction": "vertical",
        "Uo": 1.9,
        "alpha": 0.02,
        "Vo": 1.1,
    },
    {
        "location": "Den Haag",
        "method": "vibrate",
        "vibration_direction": "horizontal",
        "Uo": 2.6,
        "alpha": 0.02,
        "Vo": 0.8,
    },
    {
        "location": "Rotterdam",
        "method": "vibrate",
        "vibration_direction": "vertical",
        "Uo": 1.1,
        "alpha": 0.02,
        "Vo": 0.9,
    },
    {
        "location": "Rotterdam",
        "method": "vibrate",
        "vibration_direction": "horizontal",
        "Uo": 1.6,
        "alpha": 0.02,
        "Vo": 1.5,
    },
    {
        "location": "Tiel",
        "method": "vibrate",
        "vibration_direction": "vertical",
        "Uo": 1.1,
        "alpha": 0.02,
        "Vo": 0.9,
    },
    {
        "location": "Tiel",
        "method": "vibrate",
        "vibration_direction": "horizontal",
        "Uo": 1.6,
        "alpha": 0.02,
        "Vo": 1.5,
    },
]



from __future__ import annotations

from matplotlib import pyplot as plt
from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar


def _north_arrow(axes: plt.Axes) -> None:
    """Add north arrow to axes"""
    x, y, arrow_length = 0.05, 0.98, 0.1
    axes.annotate(
        "N",
        xy=(x, y),
        xytext=(x, y - arrow_length),
        arrowprops=dict(facecolor="black", width=5, headwidth=15),
        ha="center",
        va="center",
        fontsize=20,
        xycoords=axes.transAxes,
    )


def _scalebar(axes: plt.Axes) -> None:
    """Add size bar to axes"""
    scalebar = AnchoredSizeBar(
        axes.transData,
        20,
        "20 m",
        "lower left",
        pad=1,
        color="black",
        frameon=True,
        size_vertical=2,
    )

    axes.add_artist(scalebar)
